// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart' hide ButtonStyle;
import 'package:gap/gap.dart';
import 'package:wyatt_component_copy_with_extension/component_copy_with_extension.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_text.dart';
import 'package:wyatt_ui_kit/src/core/extensions/theme_extensions.dart';
import 'package:wyatt_ui_kit/src/core/helpers/linear_gradient_helper.dart';

part 'flat_button.g.dart';

@ComponentCopyWithExtension()
class FlatButton extends FlatButtonComponent with $FlatButtonCWMixin {
  const FlatButton({
    super.prefix,
    super.suffix,
    super.label,
    super.state,
    super.style,
    super.key,
  });

  @override
  FlatButtonStyle? get style => super.style as FlatButtonStyle?;

  @override
  Widget build(BuildContext context) => Material(
        child: Ink(
          decoration: BoxDecoration(
            color: style?.backgroundColor ??
                Theme.of(context).buttonTheme.colorScheme?.onPrimary,
            gradient: LinearGradient(
              colors: (style?.borderColors != null &&
                      style!.borderColors!.length >= 2)
                  ? style!.borderColors!
                  : [
                      // TODO(hpcl): change this
                      Theme.of(context).cardColor,
                      Theme.of(context).cardColor
                    ],
            ),
            boxShadow: [
              if (style?.shadow != null) ...[style!.shadow!]
            ],
            borderRadius: BorderRadius.all(
              Radius.circular(style?.radius ?? 0),
            ),
          ),
          child: ConstrainedBox(
            constraints: const BoxConstraints(
              minWidth: 88,
              minHeight: 36,
            ), // min sizes for Material buttons
            child: Padding(
              padding: EdgeInsets.all(style?.padding ?? 0),
              child: Row(
                mainAxisSize: mainAxisSize ?? MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  prefix ?? const SizedBox.shrink(),
                  Gap(style?.padding ?? 10),
                  if (label != null) ...[
                    Text(
                      label!.text,
                      style: label!.style ?? context.textTheme.titleLarge,
                    ).toGradient(
                      LinearGradientHelper.fromNullableColors(label!.gradient),
                    )
                  ],
                  Gap(style?.padding ?? 10),
                  suffix ?? const SizedBox.shrink(),
                ],
              ),
            ),
          ),
        ),
      );
}
